#!/usr/bin/env python

"""
Web interface data abstractions.

Copyright (C) 2014, 2015 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from datetime import datetime, timedelta
from imiptools.dates import get_datetime, get_start_of_day
from imiptools.period import Period

class EventPeriod(Period):

    "A simple period plus attribute details, compatible with RecurringPeriod."

    def __init__(self, start, end, start_attr=None, end_attr=None):
        Period.__init__(self, start, end)
        self.start_attr = start_attr
        self.end_attr = end_attr

    def as_tuple(self):
        return self.start, self.end, self.start_attr, self.end_attr

    def __repr__(self):
        return "EventPeriod(%r, %r, %r, %r)" % self.as_tuple()

def handle_date_control_values(values, with_time=True):

    """
    Handle date control information for the given 'values', returning a
    (datetime, attr) tuple, or None if the fields cannot be used to
    construct a datetime object.
    """

    if not values or not values["date"]:
        return None
    elif with_time:
        value = "%s%s" % (values["date"], values["time"])
        attr = {"TZID" : values["tzid"], "VALUE" : "DATE-TIME"}
        dt = get_datetime(value, attr)
    else:
        attr = {"VALUE" : "DATE"}
        dt = get_datetime(values["date"])

    if dt:
        return dt, attr

    return None

def handle_period_controls(start_values, end_values, dtend_enabled, dttimes_enabled, index=None):

    """
    Handle datetime controls for a particular period, described by the given
    'start_values' and 'end_values', with 'dtend_enabled' and
    'dttimes_enabled' affecting the usage of the provided values.

    If 'index' is specified, incorporate it into any error indicator.
    """

    t = handle_date_control_values(start_values, dttimes_enabled)
    if t:
        dtstart, dtstart_attr = t
    else:
        return None, [index is not None and ("dtstart", index) or "dtstart"]

    # Handle specified end datetimes.

    if dtend_enabled:
        t = handle_date_control_values(end_values, dttimes_enabled)
        if t:
            dtend, dtend_attr = t

            # Convert end dates to iCalendar "next day" dates.

            if not isinstance(dtend, datetime):
                dtend += timedelta(1)
        else:
            return None, [index is not None and ("dtend", index) or "dtend"]

    # Otherwise, treat the end date as the start date. Datetimes are
    # handled by making the event occupy the rest of the day.

    else:
        dtend = dtstart + timedelta(1)
        dtend_attr = dtstart_attr

        if isinstance(dtstart, datetime):
            dtend = get_start_of_day(dtend, attr["TZID"])

    if dtstart > dtend:
        return None, [
            index is not None and ("dtstart", index) or "dtstart",
            index is not None and ("dtend", index) or "dtend"
            ]

    return EventPeriod(dtstart, dtend, dtstart_attr, dtend_attr), None

# vim: tabstop=4 expandtab shiftwidth=4
